package com.xhkjedu.sexam.service.paper;

import com.xhkjedu.config.MissingParametersException;
import com.xhkjedu.sexam.mapper.exam.EBaseMapper;
import com.xhkjedu.sexam.mapper.paper.EPaperCorrectTeacherMapper;
import com.xhkjedu.sexam.mapper.system.UserMapper;
import com.xhkjedu.sexam.model.paperstudent.EPaperStudentQuestion;
import com.xhkjedu.sexam.vo.paper.EPaperSQuestionVo;
import com.xhkjedu.sexam.vo.paper.EPaperVo;
import com.xhkjedu.sexam.vo.system.UserVo;
import com.xhkjedu.utils.N_Utils;
import com.xhkjedu.vo.ResultVo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.AsyncResult;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.Future;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;
import java.util.stream.Collectors;

@Service
@Slf4j
public class EPaperCorrectTeacherService {
    //ReentrantLock(boolean) true为公平锁，默认false不公平锁，性能更好，可以插队！
    Lock lock = new ReentrantLock();
    @Resource
    private EPaperCorrectTeacherMapper correctTeacherMapper;
    @Resource
    private EBaseMapper eBaseMapper;
    @Resource
    private UserMapper userMapper;

    //待阅卷列表
    public List<Map> listPending(Integer teacherid) {
        return correctTeacherMapper.listPending(teacherid);
    }

    //已阅卷列表
    public List<Map> listMarked(Integer teacherid) {
        return correctTeacherMapper.listMarked(teacherid);
    }

    //随机获取待批试题
    public ResultVo getCorrectQuestion(Integer teacherid, Integer eptqid, String classids, Integer esid) {
        lock.lock();
        Map map = correctTeacherMapper.getExamNameByEsId(esid);//根据考试科目id获取考试名称
        try {
            //随机获取一条待批阅试题
            EPaperSQuestionVo sq = correctTeacherMapper.getCorrectQuestion(teacherid, eptqid, classids);
            if (sq == null) return new ResultVo(0, "已批阅完成");
            //修改试卷学生试题为批阅中
            correctTeacherMapper.updateCheckQuestionById(sq.getEpsqid(), teacherid);
            //试题批阅情况
            EPaperSQuestionVo q = correctTeacherMapper.getCheckQuestion(sq.getEptqid(), teacherid);
            Integer tjnum = q.getTjnum();
            Integer pynum = q.getPynum();
            map.put("sq", sq);//学生待批试题
            map.put("tjnum", tjnum);//提交人数
            map.put("pynum", pynum);//批阅人数
            map.put("wpynum", q.getWpynum());//我批阅人数
            map.put("pyrate", N_Utils.getIntegerDivideAndMulitiply(pynum, tjnum));//批阅进度
        } catch (Exception e) {
            log.error("随机获取待批试题出错", e.getMessage());
            throw new MissingParametersException("随机获取待批试题出错:" + e.getMessage());
        } finally {
            lock.unlock();
        }
        return new ResultVo(0, "获取成功", map);
    }

    //获取上一份批阅试题
    public ResultVo getPreQuestion(Integer eptqid, Integer teacherid, Integer esid) {
        EPaperSQuestionVo sq = correctTeacherMapper.getPreQuestion(eptqid, teacherid);
        if (sq == null) return new ResultVo(1, "您还没有批阅记录");
        Map map = correctTeacherMapper.getExamNameByEsId(esid);//根据考试科目id获取考试名称
        //试题批阅情况
        EPaperSQuestionVo q = correctTeacherMapper.getCheckQuestion(eptqid, teacherid);
        Integer tjnum = q.getTjnum();
        Integer pynum = q.getPynum();
        map.put("sq", sq);//学生待批试题
        map.put("tjnum", tjnum);//提交人数
        map.put("pynum", pynum);//批阅人数
        map.put("wpynum", q.getWpynum());//我批阅人数
        map.put("pyrate", N_Utils.getIntegerDivideAndMulitiply(pynum, tjnum));//批阅进度
        return new ResultVo(0, "获取成功", map);
    }

    //阅卷记录
    public List<Map> listRecord(Integer eptqid, Integer teacherid) {
        return correctTeacherMapper.listRecord(eptqid, teacherid);
    }

    //批阅试题
    @Async("asyncPoolTaskExecutor")
    public Future<ResultVo> correctQuestion(EPaperStudentQuestion psq) {
        try {
            correctTeacherMapper.correctQuestion(psq);
            Integer epsid = psq.getEpsid();
            Integer num = correctTeacherMapper.getNotCorrect(epsid);//学生未批改试题数量
            if (num == 0) {//批阅完成修改学生试卷批阅状态
                correctTeacherMapper.updateCheckStuById(epsid, psq.getCheckid());
            }
            return new AsyncResult<>(new ResultVo(0, "批阅成功"));
        } catch (Exception e) {
            return new AsyncResult<>(new ResultVo(1, e.getMessage()));
        }
    }

    //阅卷进度
    public Map markProgress(Integer examid) {
        Map map = eBaseMapper.findById2(examid);
        List<EPaperVo> papers = correctTeacherMapper.listPaperByExamId(examid);//获取考试试卷
        String epids = papers.stream().map(p -> p.getEpid().toString()).collect(Collectors.joining(","));
        List<EPaperStudentQuestion> questions = correctTeacherMapper.listStuQuestionByEpids(epids);//获取考试学生试题

        List<Integer> userids = questions.stream().map(q -> q.getCheckid()).distinct().collect(Collectors.toList());
        Map<Integer, String> teacherMap = null;
        if (N_Utils.isListNotEmpty(userids)) {
            List<UserVo> teachers = userMapper.listNameByUserIds(userids);//根据用户ids查找用户姓名
            teacherMap = teachers.stream().collect(Collectors.toMap(u -> u.getUserid(), u -> u.getUsername()));
        }
        for (EPaperStudentQuestion q : questions) {
            if (q.getChecked() != 2) continue;
            q.setCheckname(teacherMap.get(q.getCheckid()));//批阅人
        }

        for (EPaperVo paper : papers) {
            //单科统计
            Integer epid = paper.getEpid();
            List<EPaperStudentQuestion> qlist =
                    questions.stream().filter(q -> q.getEpid().equals(epid)).collect(Collectors.toList());
            if (N_Utils.isListEmpty(qlist)) {
                List<Map> questionList = correctTeacherMapper.listQnByEpId(epid);//获取试卷试题题号
                for (Map q : questionList) {
                    q.put("stunum ", 0);
                    q.put("pynum", 0);
                    q.put("pyrate", 0);
                    q.put("checkname", "");
                }
                paper.setQuestions(questionList);
                continue;
            }
            List<Integer> qtlist = qlist.stream().map(q -> q.getEptqid()).distinct().collect(Collectors.toList());
            List<EPaperStudentQuestion> pylist =
                    qlist.stream().filter(q -> q.getChecked().equals(2)).collect(Collectors.toList());
            Integer zgtnum = qtlist.size();
            Integer stunum = qlist.stream().map(q -> q.getStudentid()).distinct().collect(Collectors.toList()).size();
            Integer zpynum = pylist.size();//试卷批阅试题数量
            paper.setZgtnum(zgtnum);//主观题数量
            paper.setStunum(stunum);//实际人数
            paper.setPyrate(N_Utils.getIntegerDivideAndMulitiply(zpynum, qlist.size()));//批阅进度

            //单题统计
            List<Map> questionList = new ArrayList<>();
            for (Integer eptqid : qtlist) {
                List<EPaperStudentQuestion> qslist =
                        qlist.stream().filter(q -> q.getEptqid().equals(eptqid)).collect(Collectors.toList());
                stunum = qslist.size();
                Integer pynum = qslist.stream().filter(q -> q.getChecked().equals(2)).collect(Collectors.toList()).size();
                Map question = new HashMap();
                question.put("qn", qslist.get(0).getQn());//题号
                question.put("eptqid", eptqid);//试卷试题id
                question.put("pynum", pynum);//已阅题数
                question.put("stunum ", stunum);//参与人数
                question.put("pyrate", N_Utils.getIntegerDivideAndMulitiply(pynum, stunum));//批阅进度
                String checkname = qslist.stream().filter(q -> N_Utils.isNotEmpty(q.getCheckname()))
                        .map(q -> q.getCheckname()).distinct().sorted().collect(Collectors.joining("、"));
                question.put("checkname", checkname);//阅卷教师
                questionList.add(question);
            }
            paper.setQuestions(questionList);

            //阅卷教师统计
            List<Map> teacherList = new ArrayList<>();
            Map<Integer, List<EPaperStudentQuestion>> tcollect =
                    pylist.stream().filter(q -> q.getCheckid() != null).collect(Collectors.groupingBy(q -> q.getCheckid()));
            for (Map.Entry<Integer, List<EPaperStudentQuestion>> entry : tcollect.entrySet()) {
                List<EPaperStudentQuestion> list = entry.getValue();
                stunum = list.stream().map(q -> q.getStudentid()).distinct().collect(Collectors.toList()).size();
                Integer wpynum = list.size();
                Map teacher = new HashMap();
                teacher.put("checkname", list.get(0).getCheckname());//教师姓名
                teacher.put("stunum ", stunum);//参与人数
                teacher.put("pyrate", N_Utils.getIntegerDivideAndMulitiply(wpynum, zpynum));//批阅进度
                String qn = list.stream().map(q -> q.getQn()).distinct().collect(Collectors.joining(","));
                teacher.put("qn", qn);//阅卷题号
                teacherList.add(teacher);
            }
            paper.setTeachers(teacherList);
        }
        map.put("papers", papers);
        return map;
    }
}
