package com.xhkjedu.sexam.service.paperstudent;

import com.alibaba.fastjson.JSON;
import com.xhkjedu.sexam.mapper.exam.EBaseMapper;
import com.xhkjedu.sexam.mapper.exam.EClassMapper;
import com.xhkjedu.sexam.mapper.paper.EPaperMapper;
import com.xhkjedu.sexam.mapper.paperstudent.EScanbatchMapper;
import com.xhkjedu.sexam.mapper.paperstudent.EScanerrorMapper;
import com.xhkjedu.sexam.model.paperstudent.EScanbatch;
import com.xhkjedu.sexam.model.paperstudent.EScanerror;
import com.xhkjedu.sexam.utils.ExamUtil;
import com.xhkjedu.utils.N_Utils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description: TODO
 * @Author: wn
 * @CreateTime: 2022-08-11  09:31
 */
@Service
public class EScanbatchService {
    @Resource
    private EScanbatchMapper eScanbatchMapper;
    @Resource
    private EPaperMapper ePaperMapper;
    @Resource
    private EBaseMapper eBaseMapper;

    @Resource
    private EClassMapper eClassMapper;
    @Resource
    private EScanerrorMapper eScanerrorMapper;

    /**
     * @description: 保持扫描试卷批次信息
     * @param: [eScanbatch]
     * @return: java.util.Map
     * @author: wn
     * @date: 2022/8/11/011 9:42
     **/
    public Map saveScanBatch(EScanbatch eScanbatch) {
        String batchname = eScanbatch.getDevice() + "-" + eScanbatch.getEpid() + "-";
        //获取数据库中该批次名的扫描批次数量
        Integer bnum = eScanbatchMapper.getBatchNameLikeNum(batchname);
        if (bnum == null || bnum == 0) {
            bnum = 1;
        } else {
            bnum++;
        }
        batchname = batchname + bnum;
        //对应考试id
        Integer examid = ePaperMapper.getExamIdByEpId(eScanbatch.getEpid());
        eScanbatch.setBatchname(batchname);
        eScanbatch.setExamid(examid);
        eScanbatch.setScannum(0);
        eScanbatch.setErrornum(0);
        eScanbatch.setQknum(0);
        eScanbatch.setWgnum(0);
        eScanbatch.setCreatetime(N_Utils.getSecondTimestamp());
        eScanbatchMapper.insertUseGeneratedKeys(eScanbatch);
        Integer batchid = eScanbatch.getId();
        Map rtnmap = new HashMap<>();
        rtnmap.put("batchid",batchid);
        rtnmap.put("batchname",batchname);
        return rtnmap;
    }

    //扫描答题卡列表
    public Map listScan(Integer examid) {
        Map map = eBaseMapper.findById(examid);
        List<Map> subjects = eScanbatchMapper.listScanSubject(examid);//扫描答题卡列表
        map.put("subjects", subjects);
        return map;
    }

    /**
     * @description: 扫码答卷详情
     * @param: [epid]
     * @return: java.util.Map
     * @author: wn
     * @date:
     **/
    public Map scanPaperDetail(Integer epid) {
        Map paper = ePaperMapper.getExamPaperSubjectForScan(epid);
        Integer ysrnum = Integer.parseInt(paper.get("ysrnum").toString());
        Integer scantronnum = Integer.parseInt(paper.get("scantronnum").toString());
        int ysznum = ysrnum * scantronnum;//应扫码张数
        paper.put("ysznum", ysznum);
        paper.remove("scantronnum");

        if (N_Utils.isNotEmpty(paper.get("alljson"))) {
            String alljson = paper.get("alljson").toString();
            Map pamap = JSON.parseObject(alljson, Map.class);
            List<Map> ranglist = (List<Map>) pamap.get("ranglist");
            for (Map m : ranglist) {
                String ptname = m.get("qtname").toString();
                Integer num = Integer.parseInt(m.get("num").toString());
                if ("客观题".equals(ptname)) {
                    paper.put("kgt", num);
                } else {
                    paper.put("zgt", num);
                }
            }
        } else {
            paper.put("kgt", 0);
            paper.put("zgt", 0);
        }
        paper.remove("alljson");

        List<Map> batchlist = eScanbatchMapper.listScanBatchPapers(epid);

        Map rtnmap = new HashMap();
        rtnmap.put("paper", paper);
        rtnmap.put("scans", batchlist);
        return rtnmap;
    }

    //扫描答题卡详情
    public Map getScanDetail(Integer epid, Integer examid) {
        Map map = eScanbatchMapper.listByEpId(epid);
        List<Map> classes = eClassMapper.listByExamId(examid);
        int stunum = classes.stream().mapToInt(m -> (int) m.get("classnum")).sum();
        Integer scantronnum = N_Utils.str2Int(map.get("scantronnum").toString());
        Integer jhnum = stunum * scantronnum;
        EScanbatch scanbatch = eScanbatchMapper.getScanResult(epid);//获取答题卡扫描结果
        Integer scannum = scanbatch.getScannum();
        Integer errornum = scanbatch.getErrornum();
        Integer sbnum = scannum - errornum;
        Integer qknum = scanbatch.getQknum();
        Integer wsnum = ExamUtil.subtractInt(jhnum, scannum);
        map.put("jhnum", jhnum);//计划张数
        map.put("wsnum", wsnum);//未扫张数
        map.put("ysnum", scannum);//已扫张数
        map.put("qknum", qknum);//缺考人数
        map.put("smrate", N_Utils.getIntegerDivideAndMulitiply(scannum, jhnum));//扫描进度
        map.put("sbnum", sbnum);//识别张数
        map.put("sbrate", N_Utils.getIntegerDivideAndMulitiply(sbnum, scannum));//识别比例
        map.put("classes", classes);//班级列表
        return map;
    }

    //扫描答题卡详情-学生列表
    public List<Map> listScanStu(Integer epid, Integer sstate, String username, Integer classid) {
        return eScanbatchMapper.listScanStu(epid, sstate, username, classid);
    }

    //获取扫描答卷识别异常详情
    public Map getScanErrorDetail(Integer epid) {
        Map map = eScanbatchMapper.listByEpId(epid);
        List<Map> scanbatches = eScanbatchMapper.listScanBatchPapers(epid);//试卷扫描批次信息
        Map<Object, String> batchMap = scanbatches.stream().collect(Collectors.toMap(s -> s.get("batchid"), s -> s.get(
                "batchname").toString()));
        String batchids = scanbatches.stream().map(s -> s.get("batchid").toString()).collect(Collectors.joining(","));
        List<EScanerror> scanerrors = eScanerrorMapper.listByBatchIds(batchids);//试卷扫描出错列表
        scanerrors.forEach(e -> {
            e.setBatchname(batchMap.get(e.getBatchid()));
        });
        List<EScanerror> smccList = new ArrayList<>();//扫描出错集合
        List<EScanerror> khywList = new ArrayList<>();//考号有误集合
        List<EScanerror> zbdxsList = new ArrayList<>();//找不到学生集合
        List<EScanerror> qsList = new ArrayList<>();//缺少考号或试卷id集合
        Map<Integer, List<EScanerror>> collect = scanerrors.stream().collect(Collectors.groupingBy(s -> s.getErrortype()));
        for (Map.Entry<Integer, List<EScanerror>> entry : collect.entrySet()) {
            List<EScanerror> list = entry.getValue();
            Integer errortype = entry.getKey();
            if (errortype == 1) {
                smccList = list;
            } else if (errortype == 2) {
                khywList = list;
            } else if (errortype == 3) {
                zbdxsList = list;
            } else {
                qsList = list;
            }
        }
        Map smcc = new HashMap();//扫描出错
        List<EScanerror> dclList = smccList.stream().filter(s -> s.getErrorstate().equals(0)).collect(Collectors.toList());
        List<EScanerror> yclList = smccList.stream().filter(s -> s.getErrorstate().equals(1)).collect(Collectors.toList());
        smcc.put("num", dclList.size());
        smcc.put("dclList", dclList);
        smcc.put("yclList", yclList);
        map.put("smcc", smcc);
        Map khyw = new HashMap();//考号有误
        dclList = khywList.stream().filter(s -> s.getErrorstate().equals(0)).collect(Collectors.toList());
        yclList = khywList.stream().filter(s -> s.getErrorstate().equals(1)).collect(Collectors.toList());
        khyw.put("num", dclList.size());
        khyw.put("dclList", dclList);
        khyw.put("yclList", yclList);
        map.put("khyw", khyw);
        Map zbdxs = new HashMap();//找不到学生
        dclList = zbdxsList.stream().filter(s -> s.getErrorstate().equals(0)).collect(Collectors.toList());
        yclList = zbdxsList.stream().filter(s -> s.getErrorstate().equals(1)).collect(Collectors.toList());
        zbdxs.put("num", dclList.size());
        zbdxs.put("dclList", dclList);
        zbdxs.put("yclList", yclList);
        map.put("zbdxs", zbdxs);
        Map qs = new HashMap();//缺少考号或试卷id
        dclList = qsList.stream().filter(s -> s.getErrorstate().equals(0)).collect(Collectors.toList());
        yclList = qsList.stream().filter(s -> s.getErrorstate().equals(1)).collect(Collectors.toList());
        qs.put("num", dclList.size());
        qs.put("dclList", dclList);
        qs.put("yclList", yclList);
        map.put("qs", qs);
        return map;
    }

    //批量标记待处理的扫描异常为已处理
    public void batchHandleError(List<Integer> serrorids) {
        eScanerrorMapper.batchHandleError(serrorids);
    }
}
